/*
 * KernelConf - A Linux Kernel Configuration System
 *
 * Distributed under GNU General Public Lisence
 *
 * Copyright (C) 2002 by Anuradha Ratnaweera
 */

#include <ctype.h>
#include <malloc.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "../include/linux/version.h"

#include "kernelconf.h"
#include "readsymbols.h"
#include "help.h"
#include "tree.h"
#include "utils.h"

extern IdPtr id_tree_root;
//extern IdPtr first_symbol;

static char buffer[MAX_LINE_SIZE+1];
static int line_no;

static char *read_line(FILE *config_file)
{
    int i;
    char *bufptr, c;

    /* let's read the next line from CONFFILE */
    i = 0;
    while (((c = fgetc(config_file)) != EOF) && (c != '\n') && (i < MAX_LINE_SIZE))
	buffer[i++] = c;
    buffer[i] = 0;

    if ((c == EOF) && (i == 0))
	return NULL;

    line_no++;

    if (i == MAX_LINE_SIZE)
	fprintf(stderr,
		"Warning: Line with length %d read, may have been truncated\n",
		MAX_LINE_SIZE);

    /* remove comments */
    if ((bufptr = strchr(buffer, '#')))
	*bufptr = 0;

    return buffer;
}


int read_config(char* config_filename)
{
    FILE *config_file;
    char *buf, *bufptr;
    IdPtr id;

    printf ("Reading %s...", config_filename);
    if ((config_file = fopen(config_filename, "rt")) == NULL) {
	printf ("failed.\n");
	return 1;
    }

    while ((buf= read_line(config_file))) {
	if ((bufptr = strchr(buffer, '='))) {
	    buf += 7;
	    *bufptr = 0;
	    id = search_tree(&id_tree_root, buf, ID_SEARCH);
	    bufptr++;

	    if (id && id->symbol) {
		switch (id->symbol->type) {
		    case TYPE_BOOL:
		    case TYPE_TRI:
		    case TYPE_OPTION:
			if (*bufptr)
			    id->symbol->value.char_val = *bufptr;
			break;
		    case TYPE_INT:
			id->symbol->value.int_val = atoi(bufptr);
			fflush(stdout);
			break;
		    case TYPE_HEX:
			sscanf(bufptr, "%x", &(id->symbol->value.int_val));
			break;
		    case TYPE_CHAR:
			id->symbol->value.char_val = *bufptr;
			break;
		    case TYPE_STR:
			id->symbol->value.str_val = string_duplicate(buffer);
			break;
		    default:
			;
		}

	    }
	}
    }

    fclose(config_file);
    printf ("done.\n");
    return 0;
}
