/*
 * KernelConf - A Linux Kernel Configuration System
 *
 * Distributed under GNU General Public Lisence
 *
 * Copyright (C) 2002 by Anuradha Ratnaweera
 */

#include <stdio.h>

#include "kernelconf.h"
#include "writeconfig.h"

extern IdPtr first_symbol;

int write_config(char *conf_filename, char *conf_h_filename)
{
    FILE *conf, *conf_h;
    IdPtr ptr;

    if ((conf= fopen(conf_filename, "wt")) == NULL) {
	exit(1);
    }

    if ((conf_h= fopen(conf_h_filename, "wt")) == NULL) {
	exit(1);
    }

    ptr = first_symbol;

    fprintf(conf_h, "#define AUTOCONF_INCLUDED\n");

    while (ptr) {
	switch (ptr->symbol->type) {
	    case TYPE_TRI:
	    case TYPE_BOOL:
	    case TYPE_OPTION:
		if (ptr->symbol->value.char_val == 'y') {
		    fprintf(conf_h, "#define CONFIG_%s 1\n", ptr->name);
		    fprintf(conf, "CONFIG_%s=y\n", ptr->name);
		}
		else if (ptr->symbol->value.char_val == 'm') {
		    fprintf(conf_h, "#undef  CONFIG_%s\n", ptr->name);
		    fprintf(conf_h, "#define CONFIG_%s_MODULE 1\n", ptr->name);
		    fprintf(conf, "CONFIG_%s=m\n", ptr->name);
		}
		else {
		    fprintf(conf_h, "#undef  CONFIG_%s\n", ptr->name);
		    fprintf(conf, "# CONFIG_%s is not set\n", ptr->name);
		}
		break;
	    case TYPE_CHAR:
		if (1) { /* FIXME: check if the value is really valid */
		    fprintf(conf_h, "#define CONFIG_%s (%c)\n", ptr->name,
			    ptr->symbol->value.char_val);
		    fprintf(conf, "CONFIG_%s=%c\n", ptr->name,
				ptr->symbol->value.char_val);
		}
		break;
	    case TYPE_NULL:
		fprintf(conf_h, "#undef  CONFIG_%s\n", ptr->name);
		fprintf(conf, "# CONFIG_%s is not set\n", ptr->name);
		break;
	    case TYPE_INT:
		if (1) { /* FIXME: check if the value is really valid */
		    fprintf(conf_h, "#define CONFIG_%s (%d)\n", ptr->name,
			    ptr->symbol->value.int_val);
		    fprintf(conf, "CONFIG_%s=%d\n", ptr->name,
				ptr->symbol->value.int_val);
		}
		break;
	    case TYPE_HEX:
		if (1) { /* FIXME: check if the value is really valid */
		    fprintf(conf_h, "#define CONFIG_%s (%x)\n", ptr->name,
			    ptr->symbol->value.int_val);
		    fprintf(conf, "CONFIG_%s=%x\n", ptr->name,
			    ptr->symbol->value.int_val);
		}
		break;
	    case TYPE_STR:
		break;
	    default:
		;
	}

	ptr = ptr->symbol->next;
    }

    fclose(conf);
    fclose(conf_h);

    return 0;
}
